//GGX area light approximation from Horizon Zero Dawn
float GetNoHSquared(float radiusTan, float NoL, float NoV, float VoL)
{
    float radiusCos = 1.0 / sqrt(1.0 + radiusTan * radiusTan);
    
    float RoL = 2.0 * NoL * NoV - VoL;
    if(RoL >= radiusCos)
        return 1.0;

    float rOverLengthT = radiusCos * radiusTan / sqrt(1.0 - RoL * RoL);
    float NoTr = rOverLengthT * (NoV - RoL * NoL);
    float VoTr = rOverLengthT * (2.0 * NoV * NoV - 1.0 - RoL * VoL);

    float triple = sqrt(clamp(1.0 - NoL * NoL - NoV * NoV - VoL * VoL + 2.0 * NoL * NoV * VoL,0.0,1.0));
    
    float NoBr = rOverLengthT * triple, VoBr = rOverLengthT * (2.0 * triple * NoV);
    float NoLVTr = NoL * radiusCos + NoV + NoTr, VoLVTr = VoL * radiusCos + 1.0 + VoTr;
    float p = NoBr * VoLVTr, q = NoLVTr * VoLVTr, s = VoBr * NoLVTr;    
    float xNum = q * (-0.5 * p + 0.25 * VoBr * NoLVTr);
    float xDenom = p * p + s * ((s - 2.0 * p)) + NoLVTr * ((NoL * radiusCos + NoV) * VoLVTr * VoLVTr + 
                   q * (-0.5 * (VoLVTr + VoL * radiusCos) - 0.5));
    float twoX1 = 2.0 * xNum / (xDenom * xDenom + xNum * xNum);
    float sinTheta = twoX1 * xDenom;
    float cosTheta = 1.0 - twoX1 * xNum;
    NoTr = cosTheta * NoTr + sinTheta * NoBr;
    VoTr = cosTheta * VoTr + sinTheta * VoBr;
    
    float newNoL = NoL * radiusCos + NoTr;
    float newVoL = VoL * radiusCos + VoTr;
    float NoH = NoV + newNoL;
    float HoH = 2.0 * newVoL + 2.0;
    return max(0.0, NoH * NoH / HoH);
}

float GGX(vec3 n, vec3 v, vec3 l, float r, float F0, float s) {
    if(r < 0.05) r = 0.05;
    r*=r;r*=r;
    
    vec3 h = normalize(l - v);

    float dotLH = clamp(dot(h, l), 0.0, 1.0);
    float dotNL = clamp(dot(n, l), 0.0, 1.0);
    float dotNH = GetNoHSquared(s, dotNL, dot(n, -v), dot(-v, l));
    
    float denom = dotNH * r - dotNH + 1.0;
    float D = r / (3.141592653589793 * denom * denom);
    float F = exp2((-5.55473*dotLH-6.98316)*dotLH) * (1.0-F0) + F0;
    float k2 = r * 0.25;

    float specular = max(dotNL * dotNL * D * F / (dotLH * dotLH * (1.0 - k2) + k2), 0.0);
    specular = max(specular, 0.0) * (1.0 - r * (1.0 - 0.025 * F0));
    specular = specular / (0.125 * specular + 1.0);

    return specular;
}

vec3 GetMetalCol(float f0){
    int metalidx = int(f0 * 255.0);

    if(metalidx == 230) return vec3(0.24867, 0.22965, 0.21366);
    if(metalidx == 231) return vec3(0.88140, 0.57256, 0.11450);
    if(metalidx == 232) return vec3(0.81715, 0.82021, 0.83177);
    if(metalidx == 233) return vec3(0.27446, 0.27330, 0.27357);
    if(metalidx == 234) return vec3(0.84430, 0.48677, 0.22164);
    if(metalidx == 235) return vec3(0.36501, 0.35675, 0.37653);
    if(metalidx == 236) return vec3(0.42648, 0.37772, 0.31138);
    if(metalidx == 237) return vec3(0.91830, 0.89219, 0.83662);
    return vec3(1.0);
}